@@ -20,7 +20,7 @@ int main( int argc, char **argv ) {
     char *titles[6], name[32];
     int i, n = 0, col[6] = {0}, rtime = 0, old_fmt = 0, first_line;
     int old_meminfo = 0;
-    unsigned **mem;
+    long long **mem;
 
     static int sum[6]; /* statics get initialized to zero */
 
@@ -114,25 +114,25 @@ int main( int argc, char **argv ) {
                 exit (1);
             }
             printf("             total       used       free     shared    buffers     cached\n");
-            printf ("%-7s %10d %10d %10d %10d %10d %10d\n", "Mem:",
+            printf ("%-7s %10Ld %10Ld %10Ld %10Ld %10Ld %10Ld\n", "Mem:",
                     mem[meminfo_main][meminfo_total] >> byteshift,
                     mem[meminfo_main][meminfo_used] >> byteshift,
                     mem[meminfo_main][meminfo_free] >> byteshift,
                     mem[meminfo_main][meminfo_shared] >> byteshift,
                     mem[meminfo_main][meminfo_buffers] >> byteshift,
                     mem[meminfo_main][meminfo_cached] >> byteshift);
-            if (!old_fmt) printf("-/+ buffers/cache: %10d %10d\n", 
+            if (!old_fmt) printf("-/+ buffers/cache: %10Ld %10Ld\n", 
                    (mem[meminfo_main][meminfo_used]
                     -mem[meminfo_main][meminfo_buffers]
 		    -mem[meminfo_main][meminfo_cached])>> byteshift,
                    (mem[meminfo_main][meminfo_free]
                     +mem[meminfo_main][meminfo_buffers]
 		    +mem[meminfo_main][meminfo_cached])>> byteshift);
-            printf ("%-7s %10d %10d %10d\n", "Swap:",
+            printf ("%-7s %10Ld %10Ld %10Ld\n", "Swap:",
                     mem[meminfo_swap][meminfo_total] >> byteshift,
                     mem[meminfo_swap][meminfo_used] >> byteshift,
                     mem[meminfo_swap][meminfo_free] >> byteshift);
-            if (total == 1) printf("%-7s %10d %10d %10d\n", "Total:",
+            if (total == 1) printf("%-7s %10Ld %10Ld %10Ld\n", "Total:",
                        (mem[meminfo_main][meminfo_total] +
                         mem[meminfo_swap][meminfo_total]) >> byteshift,
                        (mem[meminfo_main][meminfo_used] +
