@@ -1,17 +1,21 @@
 /*
  *
- * Copyright 1994-1998 Charles Blake and Michael K. Johnson
- * This file is distributable under the conditions of the
- * GNU Library General Public License.  See the
- * file ../COPYING.LIB for details.
+ * Copyright 1994 Charles Blake and Michael K. Johnson
+ * This file is a part of procps, which is distributable
+ * under the conditions of the GNU Library General Public License.
+ * See the file COPYING for details.
  *
  */
 
 #include <string.h>		/* for strcmp */
 #include <stdio.h>		/* for parse error output */
+
 #include "proc/readproc.h"	/* for proc_t */
 #include "proc/tree.h"		/* for struct tree_node */
 
+#include "proc/compare.h"	/* for this code */
+
+
 /*
   This module was written by Charles Blake for procps.
 
@@ -49,11 +53,9 @@ or descending is the default.  That is the job of your cmp_foo's.
    It's also a little more efficient if mult_lvl_cmp() is called many times.
 */
 
-typedef int (*cmp_t)(void*,void*);	/* for function pointer casts */
-
-int sort_depth = 0;
-int sort_direction[10];     /* storage for 10 levels, but 4 would be plenty!*/
-int (*sort_function[10])(void* a, void* b);
+static int sort_depth = 0;
+static int sort_direction[10];     /* storage for 10 levels, but 4 would be plenty!*/
+static int (*sort_function[10])(void* a, void* b);
 
 int mult_lvl_cmp(void* a, void* b) {
     int i, cmp_val;
@@ -84,12 +86,12 @@ int node_mult_lvl_cmp(void* a, void* b) {
    Note the use of the string concatenation operator ##
 */
 #define CMP_STR(NAME) \
-int cmp_ ## NAME(proc_t** P, proc_t** Q) { \
+static int cmp_ ## NAME(proc_t** P, proc_t** Q) { \
     return strcmp((*P)-> ## NAME, (*Q)-> ## NAME); \
 }
 
 #define CMP_INT(NAME) \
-int cmp_ ## NAME (proc_t** P, proc_t** Q) { \
+static int cmp_ ## NAME (proc_t** P, proc_t** Q) { \
     if ((*P)-> ## NAME < (*Q)-> ## NAME) return -1; \
     if ((*P)-> ## NAME > (*Q)-> ## NAME) return  1; \
     return 0; \
@@ -100,11 +102,12 @@ int cmp_ ## NAME (proc_t** P, proc_t** Q) { \
 */
 
 /* CMP_STR(cmdline) */
-CMP_STR(user)
+CMP_STR(ruser)
+CMP_STR(euser)
 CMP_STR(cmd)
 /* CMP_INT(state) */
 /* CMP_STR(ttyc) */
-CMP_INT(uid)
+CMP_INT(euid)
 CMP_INT(pid)
 CMP_INT(ppid)
 CMP_INT(pgrp)
@@ -148,17 +151,18 @@ CMP_INT(share)
 
 /* define user interface to sort keys.  Fairly self-explanatory. */
 
-struct cmp_fun_struct {
+static struct cmp_fun_struct {
     char letter;                           /* single option-letter for key */
     char name[15];                             /* long option name for key */
     int (*fun)(proc_t**, proc_t**);  /* pointer to cmp_key */
 } cmp[] = {
 /*  { '?', "cmdline",       &cmp_cmdline       }, */
-    { 'u', "user",          &cmp_user          },
+    { 'u', "user",          &cmp_euser         },
+/*  { '?', "ruser",         &cmp_ruser         }, */
     { 'c', "cmd",           &cmp_cmd           },
 /*  { '?', "state",         &cmp_state         }, */
 /*  { '?', "ttyc",          &cmp_ttyc          }, */
-    { 'U', "uid",           &cmp_uid           },
+    { 'U', "uid",           &cmp_euid          },
     { 'p', "pid",           &cmp_pid           },
     { 'P', "ppid",          &cmp_ppid          },
     { 'g', "pgrp",          &cmp_pgrp          },
@@ -202,17 +206,6 @@ struct cmp_fun_struct {
     { '\0',"terminator",    NULL               }
 };
 
-void dump_keys(void) {
-    int i;
-    for(i=0; cmp[i].letter; i++)
-        fprintf(stderr, "%s-O%c , --sort:%-15.15s%s",
-		i%2?"":"           ",
-		cmp[i].letter, cmp[i].name,
-		i%2?"\n":"");
-    if (i%2)
-      fprintf(stderr, "\n");
-}
- 
 /* command line option parsing.  Assign sort_{depth,direction[],function[]}
    based upon a string of the form:
         [+-]a[+-]b[+-]c...
@@ -220,22 +213,25 @@ void dump_keys(void) {
    key and the optional '-' specifying a reverse sort on that key.  + doesn't
    mean anything, but it keeps things looking balanced...
 */
-int parse_sort_opt(char* opt) {
+const char *parse_sort_opt(const char* opt) {
     int i, next_dir=1;
     for(; *opt ; ++opt) {
         if (*opt == '-' || *opt == '+') {
             if (*opt == '-')
                 next_dir = -1;
+	    opt++;
             continue;
         }
         for (i = 0; cmp[i].letter; i++)
             if (*opt == cmp[i].letter)
                 break;
-        if (!cmp[i].letter) {
-	    fprintf(stderr,
-		    "ps: no such sort key -- %c.  Possibilities are:\n", *opt);
-            dump_keys();
-            return -1;
+        if (!cmp[i].letter) { /* failed, clear and return */
+            sort_depth=0;
+            for (i=0;i<10;i++){
+              sort_direction[i]=0;
+              sort_function[i]=(cmp_t)NULL;
+            }
+            return "Unknown sort key.";
         } else {
 #ifdef DEBUG
 	    fprintf(stderr,
@@ -247,10 +243,10 @@ int parse_sort_opt(char* opt) {
             next_dir = 1;
         }
     }
-    return 0;
+    return NULL;
 }
 
-int parse_long_sort(char* opt) {
+const char *parse_long_sort(const char* opt) {
     char* comma;
     int i, more_keys, next_dir=1;
     do {
@@ -268,11 +264,13 @@ int parse_long_sort(char* opt) {
         for(i = 0; cmp[i].letter; ++i)
             if (strcmp(opt, cmp[i].name) == 0)
                 break;
-        if (!cmp[i].letter) {
-            fprintf(stderr,
-		    "ps: no such sort key -- %s.  Possibilities are:\n", opt);
-            dump_keys();
-            return -1;
+        if (!cmp[i].letter) { /* failed, clear and return */
+            sort_depth=0;
+            for (i=0;i<10;i++){
+              sort_direction[i]=0;
+              sort_function[i]=(cmp_t)NULL;
+            }
+            return "Unknown sort key.";
         } else {
 #ifdef DEBUG
 	    fprintf(stderr,
@@ -285,7 +283,7 @@ int parse_long_sort(char* opt) {
         }
         opt = comma + 1; /* do next loop on next key, if more keys, else done*/
     } while (more_keys);
-    return 0;
+    return NULL;
 }
 
 void reset_sort_options (void)
