@@ -93,6 +93,7 @@ static int driver_name(char * const buf, int maj, int min){
 static int guess_name(char * const buf, int maj, int min){
   struct stat sbuf;
   int t0, t1;
+  int tmpmin = min;
   switch(maj){
   case   4:
     if(min<64){
@@ -103,10 +104,10 @@ static int guess_name(char * const buf, int maj, int min){
       sprintf(buf, "/dev/ttyS%d", min-64);
       break;
     }
-    min &= 0x3f;  /* FALL THROUGH */
+    tmpmin = min & 0x3f;  /* FALL THROUGH */
   case   3:      /* /dev/[pt]ty[p-za-o][0-9a-z] is 936 */
-    t0 = "pqrstuvwxyzabcde"[min>>4];
-    t1 = "0123456789abcdef"[min&0x0f];
+    t0 = "pqrstuvwxyzabcde"[tmpmin>>4];
+    t1 = "0123456789abcdef"[tmpmin&0x0f];
     sprintf(buf, "/dev/tty%c%c", t0, t1);
     break;
   case  17:  sprintf(buf, "/dev/ttyH%d",  min); break;
