@@ -2,32 +2,25 @@
 #include <stdio.h>
 #include <sys/types.h>
 
-/* get definition of HZ */
-/*#include <asm/param.h>*/
+/* The HZ constant from <asm/param.h> is replaced by the Hertz variable
+ * available from "proc/sysinfo.h".
+ */
 
 /* get page info */
 #include <asm/page.h>
 
-char *wchan(unsigned long);
-char *find_func(unsigned long address);
-void *xcalloc(void *pointer, int size);
-void *xmalloc(unsigned int size);
 void *xrealloc(void *oldp, unsigned int size);
+void *xmalloc(unsigned int size);
+void *xcalloc(void *pointer, int size);
        
 int   mult_lvl_cmp(void* a, void* b);
 int   node_mult_lvl_cmp(void* a, void* b);
-void  dump_keys(void);
        
 char *user_from_uid(uid_t uid);
 char *group_from_gid(gid_t gid);
 
-/* int   open_sysmap(void); */
+const char * wchan(unsigned long address);
 int   open_psdb(const char *override);
-void  close_psdb(void);
-void  make_fnctbl(void);
 
 unsigned print_str    (FILE* file, char *s, unsigned max);
 unsigned print_strlist(FILE* file, char **strs, char* sep, unsigned max);
-
-/* This doesn't really exist! */
-/* unsigned snprint_strlist(char *buf, int max, char **strs, char *sep); */
