@@ -26,9 +26,11 @@
 PROCTAB* openproc(int flags, ...) {
     va_list ap;
     PROCTAB* PT = xmalloc(sizeof(PROCTAB));
-
-    if (!Do(PID) && !(PT->procfs = opendir("/proc")))
-    	return NULL;
+    
+    if (Do(PID))
+      PT->procfs = NULL;
+    else if (!(PT->procfs = opendir("/proc")))
+      return NULL;
     PT->flags = flags;
     va_start(ap, flags);		/*  Init args list */
     if (Do(PID))
@@ -206,8 +208,8 @@ int file2str(char *directory, char *what, char *ret, int cap) {
 
     sprintf(filename, "%s/%s", directory, what);
     if ( (fd       = open(filename, O_RDONLY, 0)) == -1 ) return -1;
-    if ( (num_read = read(fd, ret, cap - 1))      <= 0 ) return -1;
-    ret[num_read] = 0;
+    if ( (num_read = read(fd, ret, cap - 1))      <= 0 ) num_read = -1;
+    else ret[num_read] = 0;
     close(fd);
     return num_read;
 }
