@@ -28,6 +28,7 @@
 #include "../proc/procps.h"
 #include "../proc/version.h"
 #include "../proc/readproc.h"
+#include "../proc/sysinfo.h"
 
 #ifndef SIGCHLD
 #define SIGCHLD SIGCLD
@@ -182,6 +183,19 @@ static void check_headers(void){
   if(!head_normal) lines_to_next_header = -1; /* how UNIX does --noheader */
 }
 
+/***** fill in %CPU; not in libproc because of include_dead_children */
+static void fill_pcpu(proc_t *buf){
+  unsigned long total_time;
+  unsigned long pcpu = 0;
+  unsigned long seconds;
+
+  total_time = buf->utime + buf->stime;
+  if(include_dead_children) total_time += (buf->cutime + buf->cstime);
+  seconds = (seconds_since_boot - ((unsigned long)buf->start_time) / Hertz);
+  if(seconds) pcpu = (total_time * 1000ULL / Hertz) / seconds;
+  buf->pcpu = (pcpu > 999) ? 999 : pcpu;
+}
+
 /***** just display */
 static void simple_spew(void){
   proc_t buf;
@@ -190,7 +204,7 @@ static void simple_spew(void){
   memset(&buf, '#', sizeof(proc_t));
   /* use "ps_" prefix to catch library mismatch */
   while(ps_readproc(ptp,&buf)){
-    /* hex_dump(&buf); */
+    fill_pcpu(&buf);
     if(want_this_proc(&buf)) show_one_proc(&buf);
     /* if(buf.cmdline) free(buf.cmdline); */   /* these crash */
     /* if(buf.environ) free(buf.environ); */
@@ -312,6 +326,7 @@ static void fancy_spew(void){
   int n = 0;  /* number of processes & index into array */
   ptp = openproc(PROC_FILLBUG);
   while((retbuf = ps_readproc(ptp,NULL))){
+    fill_pcpu(retbuf);
     if(want_this_proc(retbuf))  processes[n++] = retbuf;
     else free(retbuf);
   }
@@ -348,12 +363,12 @@ int main(int argc, char *argv[]){
     default:
       sigaction(i,&sa,NULL);
     case 0:
-    case SIGINT:  /* ^C */
-    case SIGQUIT: /* ^\ */
-    case SIGPROF: /* profiling */
-    case SIGKILL: /* can not catch */
-    case SIGSTOP: /* can not catch */
-    case SIGWINCH: /* we aren't going to change output line length later... */
+    case SIGINT:   /* ^C */
+    case SIGQUIT:  /* ^\ */
+    case SIGPROF:  /* profiling */
+    case SIGKILL:  /* can not catch */
+    case SIGSTOP:  /* can not catch */
+    case SIGWINCH: /* don't care if window size changes */
     }
   } while (0);
 #endif
