@@ -421,13 +421,13 @@ static const char *verify_short_sort(const char *arg){
 
 
 /************ parse short sorting option *************/
-/* must verify with short_sort_parse() before calling this */
 static const char *short_sort_parse(sf_node *sfn){
   int direction = 0;
   const char *walk;
   int tmp;
   sort_node *snode;
   sort_node *endp;
+  const struct shortsort_struct *ss;
   walk = sfn->sf;
   for(;;){
     tmp = *walk;
@@ -442,7 +442,9 @@ static const char *short_sort_parse(sf_node *sfn){
       direction = 1;
       break;
     default:
-      snode = do_one_sort_spec(search_shortsort_array(tmp)->spec);
+      ss = search_shortsort_array(tmp);
+      if(!ss) return "Unknown sort specifier.";
+      snode = do_one_sort_spec(ss->spec);
       if(!snode) return "Unknown sort specifier.";
       snode->reverse = direction;
       endp = snode; while(endp->next) endp = endp->next;  /* find end */
@@ -579,6 +581,7 @@ int defer_sf_option(const char *arg, int source){
 void reset_sortformat(void){
   sf_list = NULL;          /* deferred sorting and formatting */
   format_list = NULL;      /* digested formatting options */
+  sort_list = NULL;        /* digested sorting options (redundant?) */
   have_gnu_sort = 0;
   already_parsed_sort = 0;
   already_parsed_format = 0;
@@ -635,7 +638,7 @@ static const char *generate_sysv_list(void){
   format_node *fn;
   if((format_modifiers & FM_y) && !(format_flags & FF_Ul))
     return "Modifier -y without format -l makes no sense.";
-  if(prefer_bsd){
+  if(prefer_bsd_defaults){
     if(format_flags) PUSH("cmd");
     else PUSH("args");
     PUSH("bsdtime");
@@ -647,8 +650,8 @@ static const char *generate_sysv_list(void){
   }
   PUSH("tname");  /* Unix98 says "TTY" here, yet "tty" produces "TT". */
   if(format_flags & FF_Uf) PUSH("stime");
-  if(format_flags & FF_Ul) PUSH("wchan");
   if(format_flags & FF_Ul){
+    PUSH("wchan");
     if(personality & PER_IRIX_l){ /* add "rss" then ':' here */
       PUSH("sgi_rss");
       fn = malloc(sizeof(format_node));
@@ -663,11 +666,9 @@ static const char *generate_sysv_list(void){
       format_list=fn;
     }
     PUSH("sz");
-  }
-  if(format_flags & FF_Ul){
     if(format_modifiers & FM_y) PUSH("rss");
     else if(personality & (PER_ZAP_ADDR|PER_IRIX_l)) PUSH("sgi_p");
-    else PUSH("addr");
+    else PUSH("addr_1");
   }
   if(format_modifiers & FM_c){
     PUSH("pri"); PUSH("class");
