@@ -1,17 +1,20 @@
 
 /*
- * Sysctl 1.0 - A utility to read and manipulate the sysctl parameters
+ * Sysctl 1.01 - A utility to read and manipulate the sysctl parameters
  *
  *
- * Copyright 1999 George Staikos
+ * "Copyright 1999 George Staikos
  * This file may be used subject to the terms and conditions of the
  * GNU General Public License Version 2, or any later version
  * at your option, as published by the Free Software Foundation.
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU General Public License for more details.
+ * GNU General Public License for more details."
  *
+ * Changelog:
+ *            v1.01:
+ *                   - added -p <preload> to preload values from a file
  */
 
 
@@ -24,7 +27,6 @@
 #include <string.h>
 #include <errno.h>
 
-
 /*
  *    Additional types we might need.
  */
@@ -38,6 +40,7 @@ static bool false = 0;
  *    Function Prototypes
  */
 int Usage(const char *name);
+void Preload(const char *filename);
 int WriteSetting(const char *setting);
 int ReadSetting(const char *setting);
 int DisplayAll(const char *path, bool ShowTableUtil);
@@ -48,6 +51,7 @@ int DisplayAll(const char *path, bool ShowTableUtil);
  */
 
 const char *PROC_PATH = "/proc/sys/";
+const char *DEFAULT_PRELOAD = "/etc/sysctl.conf";
 static bool PrintName;
 
 /* error messages */
@@ -59,6 +63,27 @@ const char *ERR_UNKNOWN_WRITING = "error: unknown error %d setting key '%s'\n";
 const char *ERR_UNKNOWN_READING = "error: unknown error %d reading key '%s'\n";
 const char *ERR_PERMISSION_DENIED = "error: permission denied on key '%s'\n";
 const char *ERR_OPENING_DIR = "error: unable to open directory '%s'\n";
+const char *ERR_PRELOAD_FILE = "error: unable to open preload file '%s'\n";
+const char *WARN_BAD_LINE = "warning: %s(%d): invalid syntax, continuing...\n";
+
+
+#define DOTSLASH(x) do{ \
+  char *p_ = (x);           \
+  for(;;){                  \
+    p_ = strchr(p_, '.');   \
+    if(!p_) break;          \
+    *p_ = '/';              \
+  }                         \
+}while(0)
+
+#define SLASHDOT(x) do{ \
+  char *p_ = (x);           \
+  for(;;){                  \
+    p_ = strchr(p_, '.');   \
+    if(!p_) break;          \
+    *p_ = '/';              \
+  }                         \
+}while(0)
 
 
 /*
@@ -69,6 +94,7 @@ int main(int argc, char **argv) {
 const char *me = (const char *)basename(argv[0]);
 bool SwitchesAllowed = true;
 bool WriteMode = false;
+const char *preloadfile = DEFAULT_PRELOAD;
 
    PrintName = true;
 
@@ -88,6 +114,15 @@ bool WriteMode = false;
               SwitchesAllowed = false;
               WriteMode = true;
            break;
+         case 'p':
+              argv++;
+              if (argv && *argv && **argv) {
+                 preloadfile = *argv;
+              } /* endif */
+
+              Preload(preloadfile);
+              return(0);
+           break;
          case 'a':
          case 'A':
               SwitchesAllowed = false;
@@ -119,14 +154,95 @@ return 0;
  *
  */
 int Usage(const char *name) {
-   printf("usage:  %s [-n] variable ...                  \
-         \n        %s [-n] -w variable=value ...         \
-         \n        %s [-n] -a                            \
-         \n        %s [-n] -A\n", name, name, name, name);
+   printf("usage:  %s [-n] variable ... \n"
+          "        %s [-n] -w variable=value ... \n" 
+          "        %s [-n] -a \n" 
+          "        %s [-n] -p <file>   (default /etc/sysctl.conf) \n"
+          "        %s [-n] -A\n", name, name, name, name, name);
 return -1;
 }  /* end Usage() */
 
 
+/*
+ *     Strip the leading and trailing spaces from a string
+ *
+ */
+char *StripLeadingAndTrailingSpaces(char *oneline) {
+char *t;
+
+if (!oneline || !*oneline)
+   return oneline;
+
+t = oneline;
+t += strlen(oneline)-1;
+
+while ((*t == ' ' || *t == '\t' || *t == '\n' || *t == '\r') && t != oneline)
+   *t-- = 0;
+
+t = oneline;
+
+while ((*t == ' ' || *t == '\t') && *t != 0)
+   t++;
+
+return t;
+} /* end StripLeadingAndTrailingSpaces() */
+
+
+
+/*
+ *     Preload the sysctl's from the conf file
+ *           - we parse the file and then reform it (strip out whitespace)
+ *
+ */
+void Preload(const char *filename) {
+FILE *fp;
+char oneline[257];
+char buffer[257];
+char *t;
+int n = 0;
+char *name, *value;
+
+   if (!filename || ((fp = fopen(filename, "r")) == NULL)) {
+      fprintf(stderr, ERR_PRELOAD_FILE, filename);
+      return;
+   } /* endif */
+
+   while (!feof(fp)) {
+      fgets(oneline, 256, fp);
+      oneline[256] = 0;
+      n++;
+      t = StripLeadingAndTrailingSpaces(oneline);
+
+      if (strlen(t) < 2)
+         continue;
+
+      if (*t == '#' || *t == ';')
+         continue;
+
+      name = strtok(t, "=");
+      if (!name || !*name) {
+         fprintf(stderr, WARN_BAD_LINE, filename, n);
+         continue;
+      } /* endif */
+
+      StripLeadingAndTrailingSpaces(name);
+
+      value = strtok(NULL, "\n\r");
+      if (!value || !*value) {
+         fprintf(stderr, WARN_BAD_LINE, filename, n);
+         continue;
+      } /* endif */
+
+      while ((*value == ' ' || *value == '\t') && *value != 0)
+         value++;
+
+      sprintf(buffer, "%s=%s", name, value);
+      WriteSetting(buffer);
+   } /* endwhile */
+
+   fclose(fp);
+} /* end Preload() */
+
 
 
 /*
@@ -139,7 +255,6 @@ const char *name = setting;
 const char *value;
 const char *equals;
 char *tmpname;
-char *cptr;
 FILE *fp;
 char *outname;
 
@@ -170,13 +285,8 @@ char *outname;
    strncpy(outname, name, (int)(equals-name)); 
    outname[equals-name] = 0;
  
-   while (cptr = strchr(tmpname, '.')) {      /* change . to / */
-      *cptr = '/';
-   } /* endwhile */
-
-   while (cptr = strchr(outname, '/')) {      /* change / to . */
-      *cptr = '.';
-   } /* endwhile */
+   DOTSLASH(tmpname); /* change . to / */
+   SLASHDOT(outname); /* change / to . */
 
    fp = fopen(tmpname, "w");
 
@@ -214,7 +324,7 @@ return rc;
  */
 int ReadSetting(const char *setting) {
 int rc = 0;
-char *tmpname, *outname, *cptr;
+char *tmpname, *outname;
 char inbuf[1025];
 const char *name = setting;
 FILE *fp;
@@ -230,13 +340,9 @@ FILE *fp;
    strcat(tmpname, name); 
    strcpy(outname, name); 
  
-   while (cptr = strchr(tmpname, '.')) {      /* change . to / */
-      *cptr = '/';
-   } /* endwhile */
 
-   while (cptr = strchr(outname, '/')) {      /* change / to . */
-      *cptr = '.';
-   } /* endwhile */
+   DOTSLASH(tmpname); /* change . to / */
+   SLASHDOT(outname); /* change / to . */
 
    fp = fopen(tmpname, "r");
 
@@ -288,8 +394,8 @@ struct stat ts;
       rc = -1;
    } else {
       readdir(dp); readdir(dp);   /* skip . and .. */
-      while (de = readdir(dp)) {
-         tmpdir = (char *)malloc(strlen(path)+strlen(de->d_name)+1);
+      while (( de = readdir(dp) )) {
+         tmpdir = (char *)malloc(strlen(path)+strlen(de->d_name)+2);
          sprintf(tmpdir, "%s%s", path, de->d_name);
          rc2 = stat(tmpdir, &ts);       /* should check this return code */
          if (rc2 != 0) {
@@ -310,6 +416,3 @@ struct stat ts;
 return rc;
 } /* end DisplayAll() */
 
-
-
-
