@@ -62,6 +62,16 @@
  * take into account that not all 20 top processes are actually shown,
  * so it can fit more onto the screen.  I think this may help the
  * 'don't show idle' mode, but I'm not sure.
+ *
+ * Modified 1997/07/27 & 1999/01/27 Tim Janik <timj@gtk.org>
+ * added `-p' option to display specific process ids.
+ * process sorting is by default disabled in this case.
+ * added `N' and `A' keys to sort the tasks Numerically by pid or
+ * sort them by Age (newest first).
+ *
+ * Modified 1999/10/22 Tim Janik <timj@gtk.org>
+ * miscellaneous minor fixes, including "usage: ..." output for
+ * unrecognized options.
  */
 
 #include <errno.h>
@@ -84,22 +94,15 @@
 #include <stdarg.h>
 #include <sys/param.h>
 
-/* need NR_TASKS */
-#include <linux/tasks.h>
-
 #include "proc/sysinfo.h"
 #include "proc/procps.h"
 #include "proc/whattime.h"
-#include "proc/signals.h"
+#include "proc/sig.h"
 #include "proc/version.h"
 #include "proc/readproc.h"
 #include "proc/status.h"
 #include "proc/devname.h"
-/* these should be in the readproc.h header or in the procps.h header */
-typedef int (*cmp_t)(void*,void*);
-extern void reset_sort_options (void);
-extern int parse_sort_opt(char* opt);
-extern void register_sort_function (int dir, cmp_t func);
+#include "proc/compare.h"
 
 #define PUTP(x) (tputs(x,1,putchar))
 #define BAD_INPUT -30
@@ -117,7 +120,7 @@ extern void register_sort_function (int dir, cmp_t func);
        * if top is in secure mode, disallow changing of the delay time between
        * screen updates.
        */
-void parse_options(char *Options, int secure)
+static void parse_options(char *Options, int secure)
 {
     int i;
     for (i = 0; i < strlen(Options); i++) {
@@ -161,6 +164,15 @@ void parse_options(char *Options, int secure)
 	    reset_sort_options();
 	    register_sort_function( -1, (cmp_t)pcpu_sort);
 	    break;
+	  case 'N':
+	    sort_type = S_NONE;
+	    reset_sort_options();
+	    break;
+	  case 'A':
+	    sort_type = S_AGE;
+	    reset_sort_options();
+	    register_sort_function( -1, (cmp_t)age_sort);
+	    break;
 	  case 't':
 	    show_stats = 0;
 	    header_lines -= 2;
@@ -205,7 +217,7 @@ void parse_options(char *Options, int secure)
  * the 'W' command. 
  */
 
-void get_options(void)
+static void get_options(void)
 {
     FILE *fp;
     char *pt;
@@ -241,9 +253,9 @@ void get_options(void)
 }
 
 /*
-     * Set up the terminal attributes.
-     */
-void setup_terminal(void)
+ * Set up the terminal attributes.
+ */
+static void setup_terminal(void)
 {
     char *termtype;
     struct termios newtty;
@@ -296,6 +308,44 @@ void setup_terminal(void)
     tcgetattr(0, &Rawtty);
 }
 
+static int parseint(const char *src, const char *err)
+{
+    char *endp;
+    int num;
+    int len;
+    num = strtol(src, &endp, 0);
+    if (*endp == '\0') return num;
+    /* also accept prefixes of: infinite, infinity, maximum, all */
+    len = strlen(src);
+    if(len<1) goto fail;
+    if(len<9 && !strncmp(src,"infinite",len)) return INT_MAX;
+    if(len<9 && !strncmp(src,"infinity",len)) return INT_MAX;
+    if(len<8 && !strncmp(src,"maximum" ,len)) return INT_MAX;
+    if(len<4 && !strncmp(src,"all"     ,len)) return INT_MAX;
+fail:
+    fprintf(stderr, err, src);
+    exit(1);
+}
+
+static double parseflt(const char *src, const char *err)
+{
+    char *endp;
+    double num;
+    int len;
+    num = strtod(src, &endp);
+    if (*endp == '\0') return num;
+    /* also accept prefixes of: infinite, infinity, maximum, all */
+    len = strlen(src);
+    if(len<1) goto fail;
+    if(len<9 && !strncmp(src,"infinite",len)) return (double)INT_MAX;
+    if(len<9 && !strncmp(src,"infinity",len)) return (double)INT_MAX;
+    if(len<8 && !strncmp(src,"maximum" ,len)) return (double)INT_MAX;
+    if(len<4 && !strncmp(src,"all"     ,len)) return (double)INT_MAX;
+fail:
+    fprintf(stderr, err, src);
+    exit(1);
+}
+
 int main(int argc, char **argv)
 {
     /* For select(2). */
@@ -309,9 +359,14 @@ int main(int argc, char **argv)
     struct sigaction sact;
 
     get_options();
+    
+    /* set to PCPU sorting */
+    register_sort_function( -1, (cmp_t)pcpu_sort);
+    
     /*
      * Parse arguments.
      */
+    (void)argc;
     argv++;
     while (*argv) {
 	cp = *argv++;
@@ -319,16 +374,10 @@ int main(int argc, char **argv)
 	    switch (*cp) {
 	      case 'd':
 	        if (cp[1]) {
-		    if (sscanf(++cp, "%f", &Sleeptime) != 1) {
-			fprintf(stderr, PROGNAME ": Bad delay time `%s'\n", cp);
-			exit(1);
-		    }
+	            Sleeptime = parseflt(++cp, PROGNAME ": Bad delay time %s'\n");
 		    goto breakargv;
 		} else if (*argv) { /* last char in an argv, use next as arg */
-		    if (sscanf(cp = *argv++, "%f", &Sleeptime) != 1) {
-			fprintf(stderr, PROGNAME ": Bad delay time `%s'\n", cp);
-			exit(1);
-		    }
+		    Sleeptime = parseflt(cp = *argv++, PROGNAME ": Bad delay time %s'\n");
 		    goto breakargv;
 		} else {
 		    fprintf(stderr, "-d requires an argument\n");
@@ -337,17 +386,11 @@ int main(int argc, char **argv)
 		break;
 	      case 'n':
 		if (cp[1]) {
-	   	    if (sscanf(++cp, "%d", &Loops) != 1) {
-			fprintf(stderr, PROGNAME ": Bad value `%s'\n", cp);
-			exit(1);
-		    }
+		    Loops = parseint(++cp, PROGNAME ": Bad value %s'\n");
 		    goto breakargv;
 		} else if (*argv) { /* last char in an argv, use next as arg */
-		    if (sscanf(cp = *argv++, "%d", &Loops) != 1) {
-			fprintf(stderr, PROGNAME ": Bad value `%s'\n", cp);
-			 exit(1);
-	 	     }
-		     goto breakargv;
+		    Loops = parseint(cp = *argv++, PROGNAME ": Bad value %s'\n");
+		    goto breakargv;
 		}
 		break;
 					
@@ -361,6 +404,37 @@ int main(int argc, char **argv)
 		    }
 		Sleeptime = 0;
 		break;
+	      case 'p':
+		if (monpids_index >= monpids_max) {
+		    fprintf(stderr, PROGNAME ": More than %u process ids specified\n",
+			    monpids_max);
+		    exit(1);
+		}
+		if (cp[1]) {
+		    if (sscanf(++cp, "%d", &monpids[monpids_index]) != 1 ||
+			monpids[monpids_index] < 0 || monpids[monpids_index] > 65535) {
+			fprintf(stderr, PROGNAME ": Bad process id `%s'\n", cp);
+			exit(1);
+		    }
+		} else if (*argv) { /* last char in an argv, use next as arg */
+		    if (sscanf(cp = *argv++, "%d", &monpids[monpids_index]) != 1 ||
+			monpids[monpids_index] < 0 || monpids[monpids_index] > 65535) {
+			fprintf(stderr, PROGNAME ": Bad process id `%s'\n", cp);
+			exit(1);
+		    }
+		} else {
+		    fprintf(stderr, "-p requires an argument\n");
+		    exit(1);
+		}
+		if (!monpids[monpids_index])
+		    monpids[monpids_index] = getpid();
+		/* default to no sorting when monitoring process ids */
+		if (!monpids_index++) {
+		    sort_type = S_NONE;
+		    reset_sort_options();
+		}
+		cp = "_";
+		break;
 	      case 'b':
 		Batch = 1;
 	        break;
@@ -374,12 +448,15 @@ int main(int argc, char **argv)
 		Noidle = 1;
 		break;
 	      case 's':
-		Secure = 1;
-		break;
+		  Secure = 1;
+		  break;
 	      case '-':
 		break;		/* Just ignore it */
 	      default:
 		fprintf(stderr, PROGNAME ": Unknown argument `%c'\n", *cp);
+		/* fall through */
+	      case 'h':
+		fprintf(stderr, "usage: " PROGNAME " -bcisqS -d delay -p pid -n iterations\n");
 		exit(1);
 	    }
 	    cp++;
@@ -387,18 +464,11 @@ int main(int argc, char **argv)
     breakargv:
     }
     
-    /* set to PCPU sorting */
-    register_sort_function( -1, (cmp_t)pcpu_sort);
-    
     /* for correct handling of some fields, we have to do distinguish 
-  * between kernel versions */
+     * between kernel versions */
     /* get kernel symbol table, if needed */
     if (!CL_wchan_nout) {
-	if (open_psdb(NULL)) {
-	    CL_wchan_nout = 1;
-	} else {
-	    psdbsucc = 1;
-	}
+	if (open_psdb(NULL)) CL_wchan_nout = 1;
     }
 
     setup_terminal();
@@ -452,12 +522,10 @@ int main(int argc, char **argv)
 	/*
 	 *  end when exiting with an error.
 	 */
-void error_end(int rno)
+static void error_end(int rno)
 {
-    if (psdbsucc)
-        close_psdb();
     if (!Batch)
-	tcsetattr(0, TCSAFLUSH, &Savetty);
+       tcsetattr(0, TCSAFLUSH, &Savetty);
     PUTP(tgoto(cm, 0, Lines - 1));
     fputs("\r\n", stdout);
     exit(rno);
@@ -465,12 +533,11 @@ void error_end(int rno)
 /*
 	 * Normal end of execution.
 	 */
-void end(int signo)
+static void end(int signo)
 {
-    if (psdbsucc)
-	close_psdb();
+    (void)signo;
     if (!Batch)
-	tcsetattr(0, TCSAFLUSH, &Savetty);
+       tcsetattr(0, TCSAFLUSH, &Savetty);
     PUTP(tgoto(cm, 0, Lines - 1));
     fputs("\r\n", stdout);
     exit(0);
@@ -479,17 +546,18 @@ void end(int signo)
 /*
 	 * SIGTSTP catcher.
 	 */
-void stop(int signo)
+static void stop(int signo)
 {
+    (void)signo;
     /* Reset terminal. */
     if (!Batch)
-	tcsetattr(0, TCSAFLUSH, &Savetty);
+       tcsetattr(0, TCSAFLUSH, &Savetty);
     PUTP(tgoto(cm, 0, Lines - 3));
     fflush(stdout);
     raise(SIGSTOP);
     /* Later... */
     if (!Batch)
-	tcsetattr (0, TCSAFLUSH, &Rawtty);
+       tcsetattr (0, TCSAFLUSH, &Rawtty);
 }
 
 /*
@@ -497,10 +565,10 @@ void stop(int signo)
        * and also catches SIGWINCHs, and adjusts Maxlines.  Basically, this is
        * the central place for window size stuff.
        */
-void window_size(int signo)
+static void window_size(int signo)
 {
     struct winsize ws;
-
+    (void)signo;
     if((ioctl(1, TIOCGWINSZ, &ws) != -1) && (ws.ws_col>73) && (ws.ws_row>7)){
 	Cols = ws.ws_col;
 	Lines = ws.ws_row;
@@ -518,7 +586,7 @@ void window_size(int signo)
 /*
        * this adjusts the lines needed for the header to the current value
        */
-int make_header(void)
+static int make_header(void)
 {
     int i, j;
 
@@ -547,7 +615,7 @@ int make_header(void)
  * returned is a statically allocated buffer, so don't expect it to
  * persist between calls.
  */
-char *getstr(void)
+static char *getstr(void)
 {
     static char line[BUFSIZ];	/* BUFSIZ from <stdio.h>; arbitrary */
     int i = 0;
@@ -566,10 +634,10 @@ char *getstr(void)
 
 
 /*
- * Get an integer from the user.  Display an error message and return BAD_INPUT
- * if it's invalid; else return the number.
+ * Get an integer from the user.  Display an error message and
+ * return BAD_INPUT if it's invalid; else return the number.
  */
-int getint(void)
+static int getint(void)
 {
     char *line;
     int i;
@@ -594,9 +662,9 @@ int getint(void)
 
 
 /*
-	 * Get a float from the user.  Just like getint().
-	 */
-float getfloat(void)
+ * Get a float from the user.  Just like getint().
+ */
+static float getfloat(void)
 {
     char *line;
     int i;
@@ -624,13 +692,13 @@ float getfloat(void)
 	 * Get a signal number or name from the user.  Return the number, or -1
 	 * on error.
 	 */
-int getsig(void)
+static int getsig(void)
 {
     char *line;
 
     /* This is easy. */
     line = getstr();
-    return (get_signal2(line));
+    return signal_name_to_number(line);
 }
 
 /*#######################################################################
@@ -642,7 +710,7 @@ int getsig(void)
  *#######################################################################
  */
 
-int time_sort (proc_t **P, proc_t **Q)
+static int time_sort (proc_t **P, proc_t **Q)
 {
     if (Cumulative) {
 	if( ((*P)->cutime + (*P)->cstime + (*P)->utime + (*P)->stime) < 
@@ -660,20 +728,27 @@ int time_sort (proc_t **P, proc_t **Q)
     return 0;
 }
 
-int pcpu_sort (proc_t **P, proc_t **Q)
+static int pcpu_sort (proc_t **P, proc_t **Q)
 {
     if( (*P)->pcpu < (*Q)->pcpu )      return -1;
     if( (*P)->pcpu > (*Q)->pcpu )      return 1;
     return 0;
 }
 
-int mem_sort (proc_t **P, proc_t **Q)
+static int mem_sort (proc_t **P, proc_t **Q)
 {
     if( (*P)->resident < (*Q)->resident )      return -1;
     if( (*P)->resident > (*Q)->resident )      return 1;  
     return 0;
 }
 
+int age_sort (proc_t **P, proc_t **Q)
+{
+    if( (*P)->start_time < (*Q)->start_time )      return -1;
+    if( (*P)->start_time > (*Q)->start_time )      return 1;
+    return 0;
+}
+
 /*#######################################################################
  *####  Routines handling the field selection/ordering screens:  ########
  *####    show_fields, change_order, change_fields               ########
@@ -687,7 +762,7 @@ int mem_sort (proc_t **P, proc_t **Q)
 	 * The description of a displayed field is marked by a leading 
 	 * asterisk (*).
 	 */
-void show_fields(void)
+static void show_fields(void)
 {
     int i, row, col;
     char *p;
@@ -708,7 +783,7 @@ void show_fields(void)
 /*
 	 * change order of displayed fields
 	 */
-void change_order(void)
+static void change_order(void)
 {
     char c, ch, *p;
     int i;
@@ -756,7 +831,7 @@ void change_order(void)
 /*
 	 * toggle displayed fields
 	 */
-void change_fields(void)
+static void change_fields(void)
 {
     int i, changed = 0;
     int row, col;
@@ -773,13 +848,13 @@ void change_fields(void)
 	PUTP(me);
 	putchar('\n');
 	PUTP(tgoto(cm, 0, 1));
-	printf("Toggle fields with a-x, any other key to return: ");
-	fflush(stdout);
 	if (!Batch) { /* should always be true, but... */
+	    printf("Toggle fields with a-x, any other key to return: ");
+	    fflush(stdout);
 	    tcsetattr(0, TCSAFLUSH, &Rawtty);
-	    read(0, &c, 1);
-	    tcsetattr(0, TCSAFLUSH, &Savetty);
 	}
+	read(0, &c, 1);
+	tcsetattr(0, TCSAFLUSH, &Savetty);
 	i = toupper(c) - 'A';
 	if (i >= 0 && i < sizeof headers / sizeof headers[0]) {
 	    row = i % (Lines - 3) + 3;
@@ -805,45 +880,40 @@ void change_fields(void)
 	Numfields = make_header();
 }
 
-/* Do the scaling stuff
-   scale_time(time,width) - interprets time in seconds, formats it to fit width
-
-   Both return pointer to static char*.
-*/
-
-char *scale_time(int time,int width) 
+/* Do the scaling stuff: interprets time in seconds, formats it to
+ * fit width, and returns pointer to static char*.
+ */
+static char *scale_time(int t,int width) 
 {
 	static char buf[100];
 
 	/* Try successively higher units until it fits */
 
-	sprintf(buf,"%d:%02d",time/60,time%60);	/* minutes:seconds */
+	sprintf(buf,"%d:%02d",t/60,t%60);	/* minutes:seconds */
 	if (strlen(buf)<=width) 
 		return buf;
 
-	time/=60;	/* minutes */
-	sprintf(buf,"%dm",time);
+	t/=60;	/* minutes */
+	sprintf(buf,"%dm",t);
 	if (strlen(buf)<=width) 
 		return buf;
 
-	time/=60;	/* hours */
-	sprintf(buf,"%dh",time);
+	t/=60;	/* hours */
+	sprintf(buf,"%dh",t);
 	if (strlen(buf)<=width) 
 		return buf;
 
-	time/=24;	/* days */
-	sprintf(buf,"%dd",time);
+	t/=24;	/* days */
+	sprintf(buf,"%dd",t);
 	if (strlen(buf)<=width) 
 		return buf;
 	
-	time/=7;	/* weeks */
-	sprintf(buf,"%dw",time);
+	t/=7;	/* weeks */
+	sprintf(buf,"%dw",t);
 	return buf;	/* this is our last try; 
 				if it still doesn't fit, too bad. */
 
-	/* :-) I suppose if someone has a SMP version of Linux with a few
-           thousand processors, they could accumulate 18 years of CPU time... */
-           
+	/* FIXME: if someone has a 16-way SMP running over a year... */
 }
 
 /*   scale_k(k,width,unit)  - interprets k as a count, formats to fit width.
@@ -851,7 +921,7 @@ char *scale_time(int time,int width)
 			    count; 2 for megabytes; 3 for gigabytes.
 */
 
-char *scale_k(int k,int width,int unit) 
+static char *scale_k(int k,int width,int unit) 
 {
 		/* kilobytes, megabytes, gigabytes, too-big-for-int-bytes */
 	static double scale[]={1024,1024*1024,1024*1024*1024,0};
@@ -889,7 +959,7 @@ char *scale_k(int k,int width,int unit)
 	/*
 	 * Displays infos for a single task
 	 */
-void show_task_info(proc_t *task, int pmem)
+static void show_task_info(proc_t *task, int pmem)
 {
     int i,j;
     unsigned int t;
@@ -1030,22 +1100,23 @@ void show_task_info(proc_t *task, int pmem)
 /*
  * This is the real program!  Read process info and display it.
  * One could differentiate options of readproctable2, perhaps it
- * would be useful to support the PROC_UID, PROC_TTY and PROC_PID
+ * would be useful to support the PROC_UID and PROC_TTY
  * as command line options.
  */
-void show_procs(void)
+static void show_procs(void)
 {
     static proc_t **p_table=NULL;
     static int proc_flags;
     int count;
     int ActualLines;
     float elapsed_time;
-    unsigned int main_mem;
     static int first=0;
 
     if (first==0) {
 	proc_flags=PROC_FILLMEM|PROC_FILLCMD|PROC_FILLUSR;
-	p_table=readproctab2(proc_flags, p_table, NULL);
+	if (monpids_index)
+	    proc_flags |= PROC_PID;
+	p_table=readproctab2(proc_flags, p_table, monpids);
 	elapsed_time = get_elapsed_time();
 	do_stats(p_table, elapsed_time, 0);
 	sleep(1);
@@ -1061,14 +1132,14 @@ void show_procs(void)
 	PUTP(top_clrtoeol);
 	putchar('\n');
     }
-    p_table=readproctab2(proc_flags, p_table, NULL);
+    p_table=readproctab2(proc_flags, p_table, monpids);
     /* Immediately find out the elapsed time for the frame. */
     elapsed_time = get_elapsed_time();
     /* Display the system stats, calculate percent CPU time
      * and sort the list. */
     do_stats(p_table, elapsed_time,1);
     /* Display the memory and swap space usage. */
-    main_mem = show_meminfo();
+    show_meminfo();
     if (strlen(Header) + 2 > Cols)
 	Header[Cols - 2] = 0;
     PUTP(mr);
@@ -1084,18 +1155,20 @@ void show_procs(void)
     ActualLines = 0;
     while ((ActualLines < Maxlines) && (p_table[count]->pid!=-1)) {
 	int pmem;
-	char stat;
+	char Stat;
 
-	stat = p_table[count]->state;
+	Stat = p_table[count]->state;
 
-	if ( (!Noidle || (stat != 'S' && stat != 'Z')) &&
+	if ( (!Noidle || (Stat != 'S' && Stat != 'Z')) &&
 	     ( (CurrUser[0] == '\0') ||
 	      (!strcmp(CurrUser,p_table[count]->euser) ) ) ) {
 
 	    /*
 	     * Show task info.
 	     */
-	    pmem = p_table[count]->resident * 1000 / (main_mem / 4096);
+	    /* FIXME: this appears to assume a 4 kB page size */
+	    /* FIXME: maybe this can overflow on huge memory systems */
+	    pmem = p_table[count]->resident * 250 / (kb_main_total);
 	    show_task_info(p_table[count], pmem);
 	    if (!Batch)
 	    	ActualLines++;
@@ -1113,78 +1186,82 @@ void show_procs(void)
  * elapsed since the last update. This is essential for computing
  * percent CPU usage.
  */
-float get_elapsed_time(void)
+static float get_elapsed_time(void)
 {
-    struct timeval time;
+    struct timeval t;
     static struct timeval oldtime;
     struct timezone timez;
     float elapsed_time;
 
-    gettimeofday(&time, &timez);
-    elapsed_time = (time.tv_sec - oldtime.tv_sec)
-	+ (float) (time.tv_usec - oldtime.tv_usec) / 1000000.0;
-    oldtime.tv_sec = time.tv_sec;
-    oldtime.tv_usec = time.tv_usec;
+    gettimeofday(&t, &timez);
+    elapsed_time = (t.tv_sec - oldtime.tv_sec)
+	+ (float) (t.tv_usec - oldtime.tv_usec) / 1000000.0;
+    oldtime.tv_sec  = t.tv_sec;
+    oldtime.tv_usec = t.tv_usec;
     return (elapsed_time);
 }
 
-
 /*
  * Reads the memory info and displays it.  Returns the total memory
  * available, for use in percent memory usage calculations.
  */
-unsigned show_meminfo(void)
+static void show_meminfo(void)
 {
-    unsigned **mem;
-
-    if (!(mem = meminfo()) ||	/* read+parse /proc/meminfo */
-	mem[meminfo_main][meminfo_total] == 0) {	/* cannot normalize mem usage */
-	fprintf(stderr, "Cannot get size of memory from /proc/meminfo\n");
-	error_end(1);
-    }
+    meminfo();	/* read+parse /proc/meminfo */
     if (show_memory) {
-	printf("Mem:  %6dK av, %6dK used, %6dK free, %6dK shrd, %6dK buff",
-	       mem[meminfo_main][meminfo_total] >> 10,
-	       mem[meminfo_main][meminfo_used] >> 10,
-	       mem[meminfo_main][meminfo_free] >> 10,
-	       mem[meminfo_main][meminfo_shared] >> 10,
-	       mem[meminfo_main][meminfo_buffers] >> 10);
+	printf(
+	    "Mem:  %6dK av, %6dK used, %6dK free, %6dK shrd, %6dK buff",
+	    kb_main_total,
+	    kb_main_used,
+	    kb_main_free,
+	    kb_main_shared,
+	    kb_main_buffers
+	);
 	PUTP(top_clrtoeol);
 	putchar('\n');
-	printf("Swap: %6dK av, %6dK used, %6dK free                %6dK cached",
-	       mem[meminfo_swap][meminfo_total] >> 10,
-	       mem[meminfo_swap][meminfo_used] >> 10,
-	       mem[meminfo_swap][meminfo_free] >> 10,
-	       mem[meminfo_total][meminfo_cached] >> 10);
+	printf(
+	    "Swap: %6dK av, %6dK used, %6dK free                %6dK cached",
+	    kb_swap_total,
+	    kb_swap_used,
+	    kb_swap_free,
+	    kb_main_cached
+	);
 	PUTP(top_clrtoeol);
 	putchar('\n');
     }
     PUTP(me);
     PUTP(top_clrtoeol);
     putchar('\n');
-    return mem[meminfo_main][meminfo_total];
 }
 
+
 /*
  * Calculates the number of tasks in each state (running, sleeping, etc.).
  * Calculates the CPU time in each state (system, user, nice, etc).
  * Calculates percent cpu usage for each task.
  */
-void do_stats(proc_t** p, float elapsed_time, int pass)
+static void do_stats(proc_t** p, float elapsed_time, int pass)
 {
     proc_t *this;
-    int index, total_time, i, n = 0;
+    int arrindex, total_time, i, n = 0;
     int sleeping = 0, stopped = 0, zombie = 0, running = 0;
     unsigned long system_ticks = 0, user_ticks = 0, nice_ticks = 0, idle_ticks = 1000;
     static int prev_count = 0;
-    int stime, utime;
-    static struct save_hist save_history[NR_TASKS];
-    struct save_hist New_save_hist[NR_TASKS];
+    int systime, usrtime;
+ 
+     /* start with one 4K page as a reasonable allocate size */
+     static int save_history_size = sizeof(struct save_hist) * 204;
+     static struct save_hist *save_history;
+     struct save_hist *New_save_hist;
+ 
+     if (!save_history)
+       save_history = xcalloc(NULL, save_history_size);
+     New_save_hist = xcalloc(NULL, save_history_size);
 
     /*
      * Make a pass through the data to get stats.
      */
-    index = 0;
+    arrindex = 0;
     while (p[n]->pid != -1) {
 	this = p[n];
 	switch (this->state) {
@@ -1208,26 +1285,30 @@ void do_stats(proc_t** p, float elapsed_time, int pass)
 
 	/*
 	 * Calculate time in this process.  Time is sum of user time
-	 * (utime) plus system time (stime).
+	 * (usrtime) plus system time (systime).
 	 */
 	total_time = this->utime + this->stime;
-	New_save_hist[index].ticks = total_time;
-	New_save_hist[index].pid = this->pid;
-	stime = this->stime;
-	utime = this->utime;
-	New_save_hist[index].stime = stime;
-	New_save_hist[index].utime = utime;
+        if (arrindex > save_history_size) {
+            save_history_size *= 2;
+            save_history = xrealloc(save_history, save_history_size);
+            New_save_hist = xrealloc(New_save_hist, save_history_size);
+        }
+	New_save_hist[arrindex].ticks = total_time;
+	New_save_hist[arrindex].pid = this->pid;
+	systime = this->stime;
+	usrtime = this->utime;
+	New_save_hist[arrindex].stime = systime;
+	New_save_hist[arrindex].utime = usrtime;
+
 	/* find matching entry from previous pass */
-	i = 0;
-	while (i < prev_count) {
+	for (i = 0; i < prev_count; i++) {
 	    if (save_history[i].pid == this->pid) {
 		total_time -= save_history[i].ticks;
-		stime -= save_history[i].stime;
-		utime -= save_history[i].utime;
+		systime -= save_history[i].stime;
+		usrtime -= save_history[i].utime;
 
-		i = NR_TASKS;
+		i = prev_count;
 	    }
-	    i++;
 	}
 
 	/*
@@ -1237,27 +1318,27 @@ void do_stats(proc_t** p, float elapsed_time, int pass)
 	if (this->pcpu > 999)
 	    this->pcpu = 999;
 
+#if 0
 	/*
 	 * Calculate time in idle, system, user and niced tasks.
 	 */
 	idle_ticks -= this->pcpu;
-	system_ticks += stime;
-	user_ticks += utime;
+	system_ticks += systime;
+	user_ticks += usrtime;
 	if (this->nice > 0)
 	    nice_ticks += this->pcpu;
+#endif
 
-	index++;
+	arrindex++;
 	n++;
-	if (n > NR_TASKS) {
-	    printf(PROGNAME ": Help!  Too many tasks!\n");
-	    end(0);
-	}
     }
 
+#if 0
     if (idle_ticks < 0)
 	idle_ticks = 0;
     system_ticks = (system_ticks * 10 * 100/Hertz) / elapsed_time;
     user_ticks = (user_ticks * 10 * 100/Hertz) / elapsed_time;
+#endif
 
     /*
      * Display stats.
@@ -1301,6 +1382,8 @@ void do_stats(proc_t** p, float elapsed_time, int pass)
 	save_history[i].stime = New_save_hist[i].stime;
 	save_history[i].utime = New_save_hist[i].utime;
     }
+    free(New_save_hist);
+
     prev_count = n;
     qsort(p, n, sizeof(proc_t*), (void*)mult_lvl_cmp);
 }
@@ -1309,7 +1392,7 @@ void do_stats(proc_t** p, float elapsed_time, int pass)
 /*
  * Process keyboard input during the main loop
  */
-void do_key(char c)
+static void do_key(char c)
 {
     int numinput, i;
     char rcfile[MAXNAMELEN];
@@ -1371,21 +1454,22 @@ void do_key(char c)
 	if (Secure)
 	    SHOWMESSAGE(("\aCan't kill in secure mode"));
 	else {
-	    int pid, signal;
+	    int pid, signo;
 	    PUTP(md);
 	    SHOWMESSAGE(("PID to kill: "));
 	    pid = getint();
 	    if (pid == BAD_INPUT)
 		break;
 	    PUTP(top_clrtoeol);
-	    SHOWMESSAGE(("Kill PID %d with signal [15]: ", pid));
+	    SHOWMESSAGE(("Kill process %d with what signal? [15] ", pid));
 	    PUTP(me);
-	    signal = getsig();
-	    if (signal == -1)
-		signal = SIGTERM;
-	    if (kill(pid, signal))
+	    signo = getsig();
+	    /* FIXME: -1 may mean an unknown signal */
+	    if (signo == -1)
+		signo = SIGTERM;
+	    if (kill(pid, signo))
 		SHOWMESSAGE(("\aKill of PID %d with %d failed: %s",
-			     pid, signal, strerror(errno)));
+			     pid, signo, strerror(errno)));
 	}
 	break;
       case 'l':
@@ -1420,7 +1504,7 @@ void do_key(char c)
       case '#':
 	printf("Processes to display (0 for unlimited): ");
 	numinput = getint();
-	if (numinput != BAD_INPUT) {
+	if (numinput != -1) {
 	    Display_procs = numinput;
 	    window_size(0);
 	}
@@ -1452,7 +1536,18 @@ void do_key(char c)
 	reset_sort_options();
 	register_sort_function(-1, (cmp_t)pcpu_sort);
 	break;
-      case 'c':
+      case 'A':
+	SHOWMESSAGE(("Sort by age"));
+	sort_type = S_AGE;
+	reset_sort_options();
+	register_sort_function(-1, (cmp_t)age_sort);
+	break;
+      case 'N':
+	SHOWMESSAGE(("Sort numerically by pid"));
+	sort_type = S_NONE;
+	reset_sort_options();
+	break;
+    case 'c':
         show_cmd = !show_cmd;
 	SHOWMESSAGE(("Show %s", show_cmd ? "command names" : "command line"));
 	break;
@@ -1558,7 +1653,7 @@ void do_key(char c)
  */
 #define Do(x) (flags & PROC_ ## x)
 
-proc_t** readproctab2(int flags, proc_t** tab, ...) {
+static proc_t** readproctab2(int flags, proc_t** tab, ...) {
     PROCTAB* PT = NULL;
     static proc_t *buff;
     int n = 0;
@@ -1577,7 +1672,14 @@ proc_t** readproctab2(int flags, proc_t** tab, ...) {
 	i = va_arg(ap, int);
 	PT = openproc(flags, u, i);
     }
-    else if (Do(PID) || Do(TTY) || Do(STAT))
+    else if (Do(PID)) {
+	PT = openproc(flags, va_arg(ap, void*)); /* assume ptr sizes same */
+	/* work around a bug in openproc() */
+	PT->procfs = NULL;
+	/* share some process time, since we skipped opendir("/proc") */
+	usleep (50*1000);
+    }
+    else if (Do(TTY) || Do(STAT))
 	PT = openproc(flags, va_arg(ap, void*)); /* assume ptr sizes same */
     else
 	PT = openproc(flags);
