@@ -19,6 +19,7 @@ float getfloat(void);
 int time_sort(proc_t **P, proc_t **Q);
 int pcpu_sort(proc_t **P, proc_t **Q);
 int mem_sort(proc_t **P, proc_t **Q);
+int age_sort(proc_t **P, proc_t **Q);
 void show_fields(void);
 void change_order(void);
 void change_fields(void);
@@ -102,12 +103,15 @@ int show_stats = 1;    /* show status summary */
 int show_memory = 1;   /* show memory summary */
 int show_loadav = 1;   /* show load average and uptime */
 int show_cmd = 1;      /* show command name instead of commandline */
+int monpids_max = 20;
+pid_t monpids[11] = { 0 };
+int monpids_index = 0;
 int Loops = -1;	       /* number of iterations. -1 loops forever */
 int Batch = 0;         /* batch mode. Collect no input, dumb output */
 
 /* sorting order: cpu%, mem, time (cumulative, if in cumulative mode) */
 enum {
-    S_PCPU, S_MEM, S_TIME
+    S_PCPU, S_MEM, S_TIME, S_AGE, S_NONE
 };
 /* default sorting by CPU% */ 
 int sort_type = S_PCPU;
@@ -170,6 +174,8 @@ m\tToggle display of memory information\n\
 t\tToggle display of summary information\n\
 k\tKill a task (with any signal)\n\
 r\tRenice a task\n\
+N\tSort by pid (Numerically)\n\
+A\tSort by age\n\
 P\tSort by CPU usage\n\
 M\tSort by resident memory usage\n\
 T\tSort by time / cumulative time\n\
