@@ -9,22 +9,24 @@
 #include "proc/devname.h"
 #include "proc/procps.h"
 #include "proc/sysinfo.h"
+#include <ctype.h>
 #include <errno.h>
+#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 #include <string.h>
-#include <unistd.h>
-#include <time.h>
-#include <utmp.h>
-#include <fcntl.h>
 #include <sys/ioctl.h>
-#include <sys/stat.h>
 #include <sys/mman.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <pwd.h>
+#include <time.h>
+#include <unistd.h>
+#include <utmp.h>
 /* #include <sys/param.h>*/	/* for HZ */
 
-int ignoreuser = 0;		/* for '-u' */
-proc_t **procs;			/* our snapshot of the process table */
+static int ignoreuser = 0;	/* for '-u' */
+static proc_t **procs;		/* our snapshot of the process table */
 
 typedef struct utmp utmp_t;
 
@@ -34,6 +36,7 @@ typedef struct utmp utmp_t;
 #   define FROM_STRING "off"
 #endif
 
+/* Uh... same thing as UT_NAMESIZE */
 #define USERSZ (sizeof u->ut_user)
 
 
@@ -87,11 +90,11 @@ static void print_time_ival7(time_t t, int centi_sec, FILE* fout) {
 }
 
 /**** stat the device file to get an idle time */
-static time_t idletime(utmp_t *u, char *tty) {
-    struct stat terminfo;
-    if (stat(tty, &terminfo) != 0)
+static time_t idletime(char *tty) {
+    struct stat sbuf;
+    if (stat(tty, &sbuf) != 0)
 	return 0;
-    return time(NULL) - terminfo.st_atime;
+    return time(NULL) - sbuf.st_atime;
 }
 
 /***** 7 character formatted login time */
@@ -134,18 +137,28 @@ static void print_logintime(time_t logt, FILE* fout) {
 static proc_t *getproc(utmp_t *u, char *tty, int *jcpu, int *found_utpid) {
     int line;
     proc_t **p, *best = NULL;
+    unsigned uid = ~0U;
 
+    if(!ignoreuser){
+      char buf[UT_NAMESIZE+1];
+      struct passwd *passwd_data;   /* pointer to static data */
+      strncpy(buf,u->ut_user,UT_NAMESIZE);
+      buf[UT_NAMESIZE] = '\0';
+      passwd_data = getpwnam(buf);
+      if(!passwd_data) return NULL;
+      uid = passwd_data->pw_uid;
+      /* OK to have passwd_data go out of scope here */
+    }
     line = tty_to_dev(tty);
     *jcpu = *found_utpid = 0;
-    for (p = procs; *p; p++) {
-	if ((**p).pid == u->ut_pid)
-	    *found_utpid = 1;
-	if ((**p).tty == line) {
-	    (*jcpu) += (**p).utime + (**p).stime;
-	    if ((ignoreuser || !strncmp((**p).euser, u->ut_user, USERSZ)) &&
-		((**p).pid == (**p).tpgid && (!best || (**p).pid > best->pid)))
-		best = *p;
-	}
+    for(p = procs; *p; p++) {
+	if((**p).pid == u->ut_pid)   *found_utpid = 1;
+	if((**p).tty != line) continue;
+        (*jcpu) += (**p).utime + (**p).stime;
+        if(!ignoreuser && uid != (**p).euid && uid != (**p).ruid) continue;
+        if((**p).pid != (**p).tpgid) continue;
+        if(best && (**p).start_time <= best->start_time) continue;
+    	best = *p;
     }
     return best;
 }
@@ -153,12 +166,13 @@ static proc_t *getproc(utmp_t *u, char *tty, int *jcpu, int *found_utpid) {
 
 /***** showinfo */
 static void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
-    int jcpu, i, ut_pid_found;
+    int jcpu, ut_pid_found;
+    unsigned i;
     char uname[USERSZ + 1] = "",
 	tty[5 + sizeof u->ut_line + 1] = "/dev/";
     proc_t *best;
 
-    for (i=0; i < sizeof u->ut_line; i++)	/* clean up tty if garbled */
+    for (i=0; i < sizeof(u->ut_line); i++)	/* clean up tty if garbled */
 	if (isalnum(u->ut_line[i]) || (u->ut_line[i]=='/'))
 	    tty[i+5] = u->ut_line[i];
 	else
@@ -180,9 +194,9 @@ static void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
 	    print_host(u->ut_host, sizeof u->ut_host);
 	print_logintime(u->ut_time, stdout);
 	if (*u->ut_line == ':')			/* idle unknown for xdm logins */
-	    printf("   ?   ");
+	    printf(" ?xdm? ");
 	else
-	    print_time_ival7(idletime(u, tty), 0, stdout);
+	    print_time_ival7(idletime(tty), 0, stdout);
 	print_time_ival7(jcpu/Hertz, (jcpu%Hertz)*(100./Hertz), stdout);
 	if (best) {
 	    int pcpu = best->utime + best->stime;
@@ -194,9 +208,9 @@ static void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
 	if (from)
 	    print_host(u->ut_host, sizeof u->ut_host);
 	if (*u->ut_line == ':')			/* idle unknown for xdm logins */
-	    printf("   ?   ");
+	    printf(" ?xdm? ");
 	else
-	    print_time_ival7(idletime(u, tty), 0, stdout);
+	    print_time_ival7(idletime(tty), 0, stdout);
     }
     fputs("  ", stdout);
     if (best) {
